#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <inttypes.h>
#include <stdio.h>
#include <assert.h>
#include <gcrypt.h>

#include "daps.h"
#include "ds_benchmark_time.h"
#define DS_ITERATIONS 50

int main(void) {
  struct vk *vk;
  struct sk *sk, *SK;

  DS_BENCHMARK_DECLS
  int i;

  assert(gcry_check_version("1.4.5"));
  gcry_control(GCRYCTL_DISABLE_SECMEM, 0);
  gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);

  DS_BENCHMARK_START
  for (i = 0; i < DS_ITERATIONS; i++) {
    assert(KGen(&vk, &sk, 512));
	}
	DS_BENCHMARK_STOP
	DS_BENCHMARK_PRINT_AVG("keygen", DS_ITERATIONS)

  printf("signature length: %lu bytes, i.e., %lu bits\n", sig_length(vk), 8 * sig_length(vk));

  char subj[] = "Subject";

  char msg1[] = "Message 1";
  char msg2[] = "Message 2";
  char msg3[] = "Message 3";

  char sigbuf1[sig_length(vk)];
  char sigbuf2[sig_length(vk)];
  char sigbuf3[sig_length(vk)];

  DS_BENCHMARK_START
  for (i = 0; i < DS_ITERATIONS; i++) {
    Sign(sk, vk, sigbuf1, subj, strlen(subj), msg1, strlen(msg1));
  }
  DS_BENCHMARK_STOP
  DS_BENCHMARK_PRINT_AVG("sign", DS_ITERATIONS)
	DS_BENCHMARK_START
	for (i = 0; i < DS_ITERATIONS; i++) {
    assert(Ver(vk, sigbuf1, subj, strlen(subj), msg1, strlen(msg1)));
	}
	DS_BENCHMARK_STOP
	DS_BENCHMARK_PRINT_AVG("verify", DS_ITERATIONS)

  Sign(sk, vk, sigbuf2, subj, strlen(subj), msg2, strlen(msg2));
  assert(Ver(vk, sigbuf2, subj, strlen(subj), msg2, strlen(msg2)));

	DS_BENCHMARK_START
	for (i = 0; i < DS_ITERATIONS; i++) {
    SK = skExtract(vk, subj, strlen(subj), sigbuf1, msg1, strlen(msg1), sigbuf2, msg2, strlen(msg2));
	}
	DS_BENCHMARK_STOP
	DS_BENCHMARK_PRINT_AVG("extract", DS_ITERATIONS)

  Sign(SK, vk, sigbuf3, subj, strlen(subj), msg3, strlen(msg3));
  assert(Ver(vk, sigbuf3, subj, strlen(subj), msg3, strlen(msg3)));
  
  vk_release(vk);
  sk_release(sk);
  sk_release(SK);

  return 0;
}
